@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.135 2004/10/11 15:53:13 rbultje Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.139 2004/11/08 21:22:30 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -416,18 +416,6 @@ bacon_video_widget_init (BaconVideoWidget *bvw)
 
 	xine_init (bvw->priv->xine);
 
-	/* Debug configuration */
-	if (gconf_client_get_bool (bvw->priv->gc, GCONF_PREFIX"/debug", NULL) == FALSE)
-	{
-		xine_engine_set_param (bvw->priv->xine,
-				XINE_ENGINE_PARAM_VERBOSITY,
-				XINE_VERBOSITY_NONE);
-	} else {
-		xine_engine_set_param (bvw->priv->xine,
-				XINE_ENGINE_PARAM_VERBOSITY,
-				XINE_VERBOSITY_DEBUG);
-	}
-
 	/* Can we play DVDs and VCDs ? */
 	autoplug_list = xine_get_autoplay_input_plugin_ids (bvw->priv->xine);
 	while (autoplug_list && autoplug_list[i])
@@ -749,6 +737,19 @@ setup_config (BaconVideoWidget *bvw)
 		return;
 	}
 
+	/* Debug configuration */
+	if (gconf_client_get_bool (bvw->priv->gc,
+				GCONF_PREFIX"/debug", NULL) == FALSE)
+	{
+		xine_engine_set_param (bvw->priv->xine,
+				XINE_ENGINE_PARAM_VERBOSITY,
+				XINE_VERBOSITY_NONE);
+	} else {
+		xine_engine_set_param (bvw->priv->xine,
+				XINE_ENGINE_PARAM_VERBOSITY,
+				0xff);
+	}
+
 	/* Proxy configuration */
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
 	{
@@ -809,11 +810,6 @@ setup_config_video (BaconVideoWidget *bvw)
 {
 	xine_cfg_entry_t entry;
 
-	/* Default xv colourkey */
-	bvw_config_helper_num (bvw->priv->xine, "video.xv_colorkey", 30, &entry);
-	entry.num_value = 30;
-	xine_config_update_entry (bvw->priv->xine, &entry);
-
 	/* Remove the ALSA HW mixing */
 	bvw_config_helper_num (bvw->priv->xine, "audio.alsa_hw_mixer", 0, &entry);
 	entry.num_value = 0;
@@ -1287,11 +1283,12 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 		}
 		break;
 	case XINE_MSG_LIBRARY_LOAD_ERROR:
-		num = BVW_ERROR_PLUGIN_LOAD;
 		params = (char *) data + data->parameters;
+		if (bacon_video_widget_plugin_exists (bvw, params) == FALSE)
+			return;
 		/* Only if the file could really not be loaded */
-		if (bacon_video_widget_plugin_exists (bvw, params) != FALSE)
-			message = g_strdup_printf (_("A problem occured while loading a library or a decoder (%s)."), params);
+		num = BVW_ERROR_PLUGIN_LOAD;
+		message = g_strdup_printf (_("A problem occured while loading a library or a decoder (%s)."), params);
 		break;
 	case XINE_MSG_ENCRYPTED_SOURCE:
 		if (g_str_has_prefix (bvw->priv->mrl, "dvd:") != FALSE)
@@ -1863,10 +1860,62 @@ bacon_video_widget_get_backend_name (BaconVideoWidget *bvw)
 			XINE_SUB_VERSION);
 }
 
+/* List from xine-lib's demux_sputext.c */
+static const char *subtitle_ext[] = {
+	"asc",
+	"txt",
+	"sub",
+	"srt",
+	"smi",
+	"ssa"
+};
+
+static char *
+bacon_video_widget_get_subtitle_mrl (const char *mrl)
+{
+	char *suffix, *subtitle, *full;
+	guint len, i;
+
+	if (g_str_has_prefix (mrl, "file://") == FALSE) {
+		return NULL;
+	}
+
+	len = strlen (mrl);
+	if (mrl[len-4] != '.') {
+		return NULL;
+	}
+
+	full = NULL;
+	subtitle = g_strdup (mrl);
+	suffix = subtitle + len - 4;
+	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
+		char *fname;
+
+		memcpy (suffix + 1, subtitle_ext[i], 3);
+		fname = g_filename_from_uri (subtitle, NULL, NULL);
+
+		if (fname == NULL)
+			continue;
+
+		if (g_file_test (fname, G_FILE_TEST_IS_REGULAR) == FALSE) {
+			g_free (fname);
+			continue;
+		}
+
+		full = g_strdup_printf ("%s#subtitle:%s", mrl,
+				subtitle + strlen ("file://"));
+		break;
+	}
+	g_free (subtitle);
+
+	return full;
+}
+
 gboolean
 bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 		GError **error)
 {
+	char *subtitled;
 	int err;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
@@ -1874,7 +1923,15 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
-	err = xine_open (bvw->priv->stream, mrl);
+	subtitled = bacon_video_widget_get_subtitle_mrl (mrl);
+	if (subtitled != NULL) {
+		err = xine_open (bvw->priv->stream, subtitled);
+	} else {
+		err = xine_open (bvw->priv->stream, mrl);
+	}
+
+	g_free (subtitled);
+
 	if (err == 0)
 	{
 		bacon_video_widget_close (bvw);
