@@ -1,7 +1,7 @@
 /* 
    arch-tag: Implementation of Rhythmbox playlist parser
 
-   Copyright (C) 2002, 2003 Bastien Nocera
+   Copyright (C) 2002, 2003, 2004 Bastien Nocera
    Copyright (C) 2003,2004 Colin Walters <walters@rhythmbox.org>
 
    The Gnome Library is free software; you can redistribute it and/or
@@ -26,6 +26,7 @@
 #include "totem-pl-parser.h"
 
 #include "totemplparser-marshal.h"
+#include "totem-disc.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -41,6 +42,7 @@
 
 #define READ_CHUNK_SIZE 8192
 #define MIME_READ_CHUNK_SIZE 1024
+#define RECURSE_LEVEL_MAX 4
 
 typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, const char *url, gpointer data);
 static gboolean totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, const char *url);
@@ -178,11 +180,19 @@ totem_pl_parser_base_url (const char *url)
 	char *base;
 
 	uri = gnome_vfs_uri_new (url);
+	if (uri == NULL)
+		return NULL;
+
 	parent = gnome_vfs_uri_get_parent (uri);
+	if (!parent) {
+		parent = uri;
+	}
 	base = gnome_vfs_uri_to_string (parent, 0);
 
 	gnome_vfs_uri_unref (uri);
-	gnome_vfs_uri_unref (parent);
+	if (parent != uri) {
+		gnome_vfs_uri_unref (parent);
+	}
 
 	return base;
 }
@@ -246,6 +256,9 @@ totem_pl_parser_relative (const char *url, const char *output)
 
 	base = NULL;
 	url_base = totem_pl_parser_base_url (url);
+	if (url_base == NULL)
+		return NULL;
+
 	output_base = totem_pl_parser_base_url (output);
 
 	needle = strstr (url_base, output_base);
@@ -634,6 +647,8 @@ totem_pl_parser_finalize (GObject *object)
 	g_return_if_fail (parser->priv != NULL);
 
 	g_list_foreach (parser->priv->ignore_schemes, (GFunc) g_free, NULL);
+	g_free (parser->priv);
+	parser->priv = NULL;
 
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -1270,8 +1285,21 @@ static TotemPlParserResult
 totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			   gpointer data)
 {
+	MediaType type;
 	GList *list, *l;
 	GnomeVFSResult res;
+	char *media_url;
+
+	if (parser->priv->recurse_level == 1) {
+		type = cd_detect_type_from_dir (url, &media_url, NULL);
+		if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
+			if (media_url != NULL) {
+				totem_pl_parser_add_one_url (parser, media_url, NULL);
+				g_free (media_url);
+				return TOTEM_PL_PARSER_RESULT_SUCCESS;
+			}
+		}
+	}
 
 	res = gnome_vfs_directory_list_load (&list, url,
 			GNOME_VFS_FILE_INFO_DEFAULT);
@@ -1391,7 +1419,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	gpointer data = NULL;
 	gboolean ret = FALSE;
 
-	if (parser->priv->recurse_level > 4)
+	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	parser->priv->recurse_level++;
