@@ -18,17 +18,17 @@
  */
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
 #include <gtk/gtkmessagedialog.h>
 #include <string.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <bacon-cd-selection.h>
 
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
-#include "bacon-cd-selection.h"
+#include "video-utils.h"
 
 #include "debug.h"
 
@@ -50,6 +50,7 @@ totem_action_info (char *reason, Totem *totem)
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
 				"%s", reason);
+	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
 			GTK_RESPONSE_OK);
 	g_signal_connect (G_OBJECT (error_dialog), "destroy", G_CALLBACK
@@ -61,35 +62,6 @@ totem_action_info (char *reason, Totem *totem)
 	gtk_widget_show (error_dialog);
 }
 
-
-static gboolean
-totem_display_is_local (Totem *totem)
-{
-	const char *name, *work;
-	int display, screen;
-
-	name = gdk_display_get_name (gdk_display_get_default ());
-	if (name == NULL)
-		return TRUE;
-
-	work = strstr (name, ":");
-	if (work == NULL)
-		return TRUE;
-
-	/* Get to the character after the colon */
-	work++;
-	if (work == NULL)
-		return TRUE;
-
-	if (sscanf (work, "%d.%d", &display, &screen) != 2)
-		return TRUE;
-
-	if (display < 10)
-		return TRUE;
-
-	return FALSE;
-}
-
 static void
 hide_prefs (GtkWidget *widget, int trash, Totem *totem)
 {
@@ -110,6 +82,7 @@ ask_show_visuals (Totem *totem)
 				_("It seems you are running Totem remotely.\n"
 				"Are you sure you want to enable the visual "
 				"effects?"));
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 			GTK_RESPONSE_NO);
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -173,7 +146,7 @@ on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
 
 	value = gtk_toggle_button_get_active (togglebutton);
 
-	if (value == TRUE && totem_display_is_local (totem) == FALSE)
+	if (value != FALSE && totem_display_is_local () == FALSE)
 	{
 		if (ask_show_visuals (totem) == FALSE)
 		{
@@ -202,7 +175,7 @@ on_tvout_toggled (GtkToggleButton *togglebutton, Totem *totem)
 	value = bacon_video_widget_set_tv_out
 		(BACON_VIDEO_WIDGET (totem->bvw), type);
 
-	if (value == TRUE)
+	if (value != FALSE)
 		totem_action_info (_("Switching on or off this type of TV-Out requires a restart to take effect."), totem);
 }
 
@@ -326,7 +299,7 @@ on_button1_clicked (GtkButton *button, Totem *totem)
 	path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
 
-	if (g_getenv ("KDE_STARTUP_ENV") && (filemanager = g_find_program_in_path("konqueror"))) {
+	if ((g_getenv ("KDE_STARTUP_ENV") || g_getenv("KDE_FULL_SESSION")) && (filemanager = g_find_program_in_path("konqueror"))) {
 		cmd = g_strconcat (filemanager, " ", path, NULL);
 		g_free (filemanager);
 	} else {
@@ -363,7 +336,7 @@ visual_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
 		gconf_client_set_string (totem->gc, GCONF_PREFIX"/visual",
 				name, NULL);
 
-		if (bacon_video_widget_set_visuals (totem->bvw, name) == TRUE)
+		if (bacon_video_widget_set_visuals (totem->bvw, name) != FALSE)
 			totem_action_info (_("Changing the visuals effect type will require a restart to take effect."), totem);
 	}
 }
@@ -410,6 +383,44 @@ contrast_changed (GtkRange *range, Totem *totem)
 			BVW_VIDEO_CONTRAST, (int) i);
 }
 
+static void
+saturation_changed (GtkRange *range, Totem *totem)
+{
+	gdouble i;
+
+	i = gtk_range_get_value (range);
+	bacon_video_widget_set_video_property (totem->bvw,
+			BVW_VIDEO_SATURATION, (int) i);
+}
+
+static void
+hue_changed (GtkRange *range, Totem *totem)
+{
+	gdouble i;
+
+	i = gtk_range_get_value (range);
+	bacon_video_widget_set_video_property (totem->bvw,
+			BVW_VIDEO_HUE, (int) i);
+}
+
+static void
+on_tpw_color_reset_clicked (GtkButton *button, Totem *totem)
+{
+	guint i;
+	char *scales[] = {
+		"tpw_bright_scale",
+		"tpw_contrast_scale",
+		"tpw_saturation_scale",
+		"tpw_hue_scale"
+	};
+
+	for (i = 0; i < G_N_ELEMENTS (scales); i++) {
+		GtkWidget *item;
+		item = glade_xml_get_widget (totem->xml, scales[i]);
+		gtk_range_set_value (GTK_RANGE (item), 65535/2);
+	}
+}
+
 static void
 audio_out_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
 {
@@ -431,7 +442,7 @@ totem_setup_preferences (Totem *totem)
 
 	g_return_if_fail (totem->gc != NULL);
 
-	is_local = totem_display_is_local (totem);
+	is_local = totem_display_is_local ();
 
 	gconf_client_add_dir (totem->gc, "/apps/totem",
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -469,23 +480,20 @@ totem_setup_preferences (Totem *totem)
 	if (mediadev == NULL || (strcmp (mediadev, "") == 0)
 			|| (strcmp (mediadev, "auto") == 0))
 	{
-		mediadev = bacon_cd_selection_get_default_device
-			(BACON_CD_SELECTION (item));
+		mediadev = g_strdup (bacon_cd_selection_get_default_device
+				     (BACON_CD_SELECTION (item)));
 		gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev",
 				mediadev, NULL);
-		bacon_video_widget_set_media_device
-			(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
-	} else {
-		bacon_video_widget_set_media_device
-			(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
 	}
+	bacon_video_widget_set_media_device
+		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
 
-	bacon_cd_selection_set_device (BACON_CD_SELECTION (item),
-			gconf_client_get_string
-			(totem->gc, GCONF_PREFIX"/mediadev", NULL));
 	g_signal_connect (G_OBJECT (item), "device-changed",
 			G_CALLBACK (on_combo_entry1_changed), totem);
 
+	bacon_cd_selection_set_device (BACON_CD_SELECTION (item), mediadev);
+	g_free (mediadev);
+
 	/* Connection Speed */
 	connection_speed = bacon_video_widget_get_connection_speed (totem->bvw);
 	item = glade_xml_get_widget (totem->xml, "tpw_speed_optionmenu");
@@ -509,7 +517,7 @@ totem_setup_preferences (Totem *totem)
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_checkbutton");
 	show_visuals = gconf_client_get_bool (totem->gc,
 			GCONF_PREFIX"/show_vfx", NULL);
-	if (is_local == FALSE && show_visuals == TRUE)
+	if (is_local == FALSE && show_visuals != FALSE)
 		show_visuals = ask_show_visuals (totem);
 
 	gtk_toggle_button_set_active
@@ -542,6 +550,7 @@ totem_setup_preferences (Totem *totem)
 
 		i++;
 	}
+	g_free (visual);
 
 	/* Visualisation quality */
 	i = gconf_client_get_int (totem->gc,
@@ -574,6 +583,27 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "value-changed",
 			G_CALLBACK (contrast_changed), totem);
 
+	/* Saturation */
+	item = glade_xml_get_widget (totem->xml, "tpw_saturation_scale");
+	i = bacon_video_widget_get_video_property (totem->bvw,
+			BVW_VIDEO_SATURATION);
+	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
+	g_signal_connect (G_OBJECT (item), "value-changed",
+			G_CALLBACK (saturation_changed), totem);
+
+	/* Hue */
+	item = glade_xml_get_widget (totem->xml, "tpw_hue_scale");
+	i = bacon_video_widget_get_video_property (totem->bvw,
+			BVW_VIDEO_HUE);
+	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
+	g_signal_connect (G_OBJECT (item), "value-changed",
+			G_CALLBACK (hue_changed), totem);
+
+	/* Reset colour balance */
+	item = glade_xml_get_widget (totem->xml, "tpw_color_reset");
+	g_signal_connect (G_OBJECT (item), "clicked",
+			G_CALLBACK (on_tpw_color_reset_clicked), totem);
+
 	/* Sound output type */
 	item = glade_xml_get_widget (totem->xml, "tpw_sound_output_optionmenu");
 	audio_out = bacon_video_widget_get_audio_out_type (totem->bvw);
@@ -615,8 +645,11 @@ totem_preferences_tvout_setup (Totem *totem)
 	case TV_OUT_NONE:
 		name = "tpw_notvout_radio_button";
 		break;
-	case TV_OUT_TVMODE:
-		name = "tpw_tvoutmode_radio_button";
+	case TV_OUT_NVTV_PAL:
+		name = "tpw_nvtvpalmode_radio_button";
+		break;
+	case TV_OUT_NVTV_NTSC:
+		name = "tpw_nvtvntscmode_radio_button";
 		break;
 	case TV_OUT_DXR3:
 		name = "tpw_dxr3tvout_radio_button";
@@ -634,16 +667,32 @@ totem_preferences_tvout_setup (Totem *totem)
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_NONE));
-	item = glade_xml_get_widget (totem->xml, "tpw_tvoutmode_radio_button");
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NONE));
+
+	item = glade_xml_get_widget (totem->xml, "tpw_nvtvpalmode_radio_button");
+        g_signal_connect (G_OBJECT (item), "toggled",
+                G_CALLBACK (on_tvout_toggled), totem);
+        g_object_set_data (G_OBJECT (item), "tvout_type",
+                GINT_TO_POINTER (TV_OUT_NVTV_PAL));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_PAL));
+
+	item = glade_xml_get_widget (totem->xml, "tpw_nvtvntscmode_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
-			GINT_TO_POINTER (TV_OUT_TVMODE));
+                GINT_TO_POINTER (TV_OUT_NVTV_NTSC));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_NTSC));
+	
 	item = glade_xml_get_widget (totem->xml, "tpw_dxr3tvout_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_DXR3));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_DXR3));
 }
 
 void
@@ -657,5 +706,5 @@ totem_preferences_visuals_setup (Totem *totem)
 		visual = g_strdup ("goom");
 
 	bacon_video_widget_set_visuals (totem->bvw, visual);
+	g_free (visual);
 }
-
